[ "${HOOK_LABEL}" = "" ] && HOOK_LABEL="unknown"
export LOG_DIR="var/log"
export LAT_DIR="tmp/lat"
export LOGFILE="/${LAT_DIR}/kickstart.log"
function log()
{
local dt="$(date "+%Y-%m-%d %H:%M:%S.%3N")"
echo "$dt kickstart ${HOOK_LABEL}  info: ${prefix} $1" >>${LOGFILE}
}
function ilog()
{
[ -z "${stdout}" ] && stdout=1
local dt="$(date "+%Y-%m-%d %H:%M:%S.%3N")"
echo "$dt kickstart ${HOOK_LABEL}  info: ${prefix} $1" >&${stdout}
echo "$dt kickstart ${HOOK_LABEL}  info: ${prefix} $1" >>${LOGFILE}
}
function dlog()
{
if [ ${debug} -ne 0 ] ; then
[ -z "$stdout" ] && stdout=1
local dt="$(date "+%Y-%m-%d %H:%M:%S.%3N")"
echo "$dt kickstart ${HOOK_LABEL} debug: ${prefix} $1" >&${stdout}
echo "$dt kickstart ${HOOK_LABEL} debug: ${prefix} $1" >>${LOGFILE}
fi
}
function wlog()
{
[ -z "$stdout" ] && stdout=1
local dt="$(date "+%Y-%m-%d %H:%M:%S.%3N")"
echo "$dt kickstart ${HOOK_LABEL}  warn: ${prefix} $1" >&${stdout}
echo "$dt kickstart ${HOOK_LABEL}  warn: ${prefix} $1" >>${LOGFILE}
}
function elog()
{
[ -z "$stdout" ] && stdout=1
local dt="$(date "+%Y-%m-%d %H:%M:%S.%3N")"
echo "$dt kickstart ${HOOK_LABEL} error: ${prefix} $1" >&${stdout}
echo "$dt kickstart ${HOOK_LABEL} error: ${prefix} $1" >>${LOGFILE}
}
function report_failure_with_msg()
{
local msg=${1}
echo -e '\n\nInstallation failed.\n'
wlog "Installation Failed: ERROR: ${msg}"
echo "... dropping to shell ; exit to reboot ..."
sleep 1
bash
reboot -f
}
function get_disk()
{
echo "$(cd /dev ; readlink -f $1)"
}
function get_persistent_disk()
{
local dev=$(cd /dev ; readlink -f $1)
case $dev in
*"dm-"*)
for p in /dev/disk/by-id/wwn-*; do
if [ "$(basename $dev)" = "$(basename $(readlink -f $p))" ]; then
echo "$p"
return
fi
done
;;
*)
reverse_list=""
for p in /dev/disk/by-path/*; do
reverse_list="${p} ${reverse_list}"
done
for p in ${reverse_list}; do
if [ "$(basename $dev)" = "$(basename $(readlink -f $p))" ]; then
echo "$p"
return
fi
done
;;
esac
}
function get_part_prefix()
{
local dev=$(cd /dev ; readlink -f $1)
case $dev in
*"nvme"*)
echo "${dev}p"
;;
*"dm-"*)
for p in /dev/disk/by-id/wwn-*; do
if [ "$dev" = "$(readlink -f ${p})" ]; then
echo "${p}-part"
return
fi
done
;;
*)
echo "${dev}"
;;
esac
}
function get_default_install_disk()
{
local disk
for blk_dev in vda vdb sda sdb dda ddb hda hdb; do
if [ -d /sys/block/$blk_dev ]; then
disk=$(ls -l /sys/block/$blk_dev | grep -v usb | head -n1 | sed 's/^.*\([vsdh]d[a-z]\+\).*$/\1/');
if [ -n "$disk" ]; then
multipath -c /dev/$disk > /dev/null && continue
echo "$disk"
return
fi
fi
done
for blk_dev in nvme0n1 nvme1n1; do
if [ -d /sys/block/$blk_dev ]; then
disk=$(ls -l /sys/block/$blk_dev | grep -v usb | head -n1 | sed 's/^.*\(nvme[01]n1\).*$/\1/');
if [ -n "$disk" ]; then
echo "$disk"
return
fi
fi
done
for mpath_dev in mpatha mpathb; do
if [ -e /dev/mapper/$mpath_dev ]; then
echo "/dev/mapper/$mpath_dev"
return
fi
done
}
function exec_retry()
{
local retries=${1}
local interval=${2}
local command=${3}
retry_count=1
local ret_code=0
local ret_stdout=""
command="${command}" # 2>&${stdout}"
while [ ${retry_count} -le ${retries} ]; do
ret_stdout=$(eval ${command})
ret_code=${?}
[ ${ret_code} -eq 0 ] && break
wlog "Error running command '${command}'. Try ${retry_count} of ${retries} retry in ${interval}s."
wlog "ret_code: ${ret_code}, stdout: '${ret_stdout}'."
sleep ${interval}
retry_count=$((retry_count+1))
done
echo "${ret_stdout}"
return ${ret_code}
}
function exec_no_fds()
{
local fds=${1}
local cmd=${2}
local retries=${3}
local interval=${4}
local ret_code=0
local ret_stdout=""
for fd in ${fds}
do
local cmd="${cmd} $fd>&-"
done
if [ -z "${retries}" ]; then
eval "${cmd}"
else
ret_stdout=$(exec_retry "${retries}" "${interval}" "${cmd}")
ret_code=${?}
echo "${ret_stdout}"
return ${ret_code}
fi
}
function is_system_node_install()
{
if [[ "${insturl}" == *"pxecontroller"* ]] ; then
return 0
else
return 1
fi
}
function is_usb_install()
{
pxeurl=$(echo ${insturl} | sed -e s/ostree_repo//)
if [ "${pxeurl}" = "file://NOT_SET" ] ; then
return 0
else
return 1
fi
}
function check_prestage()
{
set -- net.naming-scheme=vSTX7_0 BOOT_IMAGE=vlm-boards/yow-wildcat-111/feed/bzImage-rt rdinit=/install defaultkernel=vmlinuz-*-rt-amd64 instname=debian instbr=starlingx instab=0 traits=controller,worker,lowlatency efi=runtime BOOTIF=a4:bf:01:19:56:13 instdev=/dev/disk/by-path/pci-0000:83:00.0-nvme-1 inst_ostree_root=/dev/mapper/cgts--vg-root--lv inst_ostree_var=/dev/mapper/cgts--vg-var--lv ks=http://10.64.1.254/umalab/yow-wildcat-111_feed/kickstart/kickstart.cfg insturl=http://10.64.1.254/umalab/yow-wildcat-111_feed/ostree_repo console=ttyS0,115200 biosdevname=0 biosplusefi=1
for value in "$@"; do
if [ "${value}" = "prestage" ]; then
touch /tmp/lat/.prestage
return 0
fi
done
return 1
}
function is_prestage()
{
if [ -e /tmp/lat/.prestage ] ; then
return 0
else
return 1
fi
}
PLATFORM_CONF="/etc/platform/platform.conf"
function update_platform_conf()
{
ilog "update ${IMAGE_ROOTFS}${PLATFORM_CONF} : ${1}"
echo "${1}" >> ${IMAGE_ROOTFS}${PLATFORM_CONF}
}
function get_iface_from_ethname()
{
iface=${1}
udevadm info --export --query=property --path=/sys/class/net/${iface} >> ${LOGFILE}
altname=$(udevadm info --export --query=property --path=/sys/class/net/${iface} | grep ID_NET_NAME_ONBOARD)
altname=${altname#ID_NET_NAME_ONBOARD=}
if [ -z "${altname}" ] ; then
log "... retrying to translate ${iface} to predictive name with ID_NET_NAME_SLOT filter label "
altname=$(udevadm info --export --query=property --path=/sys/class/net/${iface} | grep ID_NET_NAME_SLOT)
altname=${altname#ID_NET_NAME_SLOT=}
fi
if [ -z "${altname}" ] ; then
log "... retrying to translate ${iface} to predictive name with ID_NET_NAME_PATH filter label "
altname=$(udevadm info --export --query=property --path=/sys/class/net/${iface} | grep ID_NET_NAME_PATH)
altname=${altname#ID_NET_NAME_PATH=}
fi
if [ -z "${altname}" ] ; then
log "Failed to translate ${iface} to predictive name"
elif [ "${altname::1}" = "'" ] ; then
echo "${altname:1:-1}"
else
echo "${altname}"
fi
}
function display_volume_info ()
{
if [ ${debug} -ne 0 ] ; then
ilog "Volume Info:${1}"
pvscan 2>/dev/null
pvscan 2>/dev/null >> ${LOGFILE}
lvscan 2>/dev/null
lvscan 2>/dev/null >> ${LOGFILE}
pvdisplay 2>/dev/null
pvdisplay 2>/dev/null >> ${LOGFILE}
lvdisplay 2>/dev/null
lvdisplay 2>/dev/null >> ${LOGFILE}
fi
}
display_mount_info()
{
if [ ${debug} -ne 0 ] ; then
echo "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~"
lsblk -o name,mountpoint,label,size,uuid
echo "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~"
fi
}
function log_lvm_conf ()
{
local msg=${1}
local conf=${2}
lvm_global_filter=$(cat ${conf} | grep "global_filter =")
lvm_preferred_names=$(cat ${conf} | grep "preferred_names =")
ilog "${msg}: ${conf}: ${lvm_global_filter}"
ilog "${msg}: ${conf}: ${lvm_preferred_names}"
}
function breakpoint()
{
echo "*** BREAKPOINT ***: ${1} ; type exit to continue"
bash
echo "continuing from breakpoint: ${1}"
}
function set_variable()
{
touch "/${LAT_DIR}/${1}"
}
function get_variable()
{
[ -e "/${LAT_DIR}/${1}" ] && return 1
return 0
}
function clr_variable()
{
rm -f "/${LAT_DIR}/${1}"
}
function get_aio_controller_provisioning_sizes()
{
gib=$((1024*1024*1024))
export PART_SZ_EFI=300
export PART_SZ_BOOT=2048
export PART_SZ_ROOT=0
export LV_SZ_LOG=8000
export LV_SZ_ROOT=20480
export LV_SZ_SCRATCH=16000
export LV_SZ_VAR=20480
default_small_disk_size=240
minimum_small_disk_size=196
sz=$(blockdev --getsize64 ${INSTDEV})
ilog "Install disk ${INSTDEV} has size:${sz}B, $((${sz}/$gib))GiB"
if [ ${sz} -gt $((${default_small_disk_size}*${gib})) ] ; then
ilog "Large disk: MINIMUM_PLATFORM_PV_SIZE=219GiB*1024=224256 MiB"
export BACKUP_DEFAULT_PERSISTENT_SIZE=30000
elif [ ${sz} -ge $((${minimum_small_disk_size}*${gib})) ] ; then
ilog "Small disk: MINIMUM_PLATFORM_PV_SIZE=164GiB*1024=167936 MiB"
export BACKUP_DEFAULT_PERSISTENT_SIZE=30000
else
ilog "Tiny disk: MINIMUM_PLATFORM_PV_SIZE=43GiB*1024=44032 MiB"
export LV_SZ_LOG=3000
export LV_SZ_SCRATCH=2000
export BACKUP_DEFAULT_PERSISTENT_SIZE=1000
fi
}
function get_std_controller_provisioning_sizes()
{
export PART_SZ_EFI=300
export PART_SZ_BOOT=2048
export PART_SZ_ROOT=0
export BACKUP_DEFAULT_PERSISTENT_SIZE=30000
export LV_SZ_LOG=8000
export LV_SZ_ROOT=20480
export LV_SZ_SCRATCH=16000
export LV_SZ_VAR=20480
}
function get_worker_provisioning_sizes()
{
gib=$((1024*1024*1024))
export PART_SZ_EFI=300
export PART_SZ_BOOT=2048
export PART_SZ_ROOT=0
export LV_SZ_LOG=4096
export LV_SZ_SCRATCH=4096
sz=$(blockdev --getsize64 ${INSTDEV})
if [ $sz -le $((120*$gib)) ] ; then
export LV_SZ_ROOT=10240
export LV_SZ_VAR=10240
else
export LV_SZ_ROOT=20480
export LV_SZ_VAR=20480
fi
ilog "Install disk ${INSTDEV} has size:${sz}B, $((${sz}/$gib))GiB"
}
function get_storage_provisioning_sizes()
{
gib=$((1024*1024*1024))
export PART_SZ_EFI=300
export PART_SZ_BOOT=2048
export PART_SZ_ROOT=0
sz=$(blockdev --getsize64 ${INSTDEV})
if [ $sz -le $((120*$gib)) ] ; then
export LV_SZ_LOG=4000
export LV_SZ_ROOT=10240
export LV_SZ_SCRATCH=4000
export LV_SZ_VAR=10240
else
export LV_SZ_LOG=8000
export LV_SZ_ROOT=20480
export LV_SZ_SCRATCH=8000
export LV_SZ_VAR=20480
fi
ilog "Install disk ${INSTDEV} has size:${sz}B, $((${sz}/$gib))GiB"
}
function parse_miniboot_network_params()
{
elog "Unused: parse_miniboot_network_params"
}
if [ -d /sys/firmware/efi ] ; then
ilog "UEFI Firmware execution"
USE_UEFI_PARTITIONS=1
else
ilog "BIOS Firmware execution"
USE_UEFI_PARTITIONS=0
fi
if [ "${controller}" = true ] ; then
BACKUP_PART_UEFI=1
BACKUP_PART_BIOS=2
if [ "$USE_UEFI_PARTITIONS" = 0 ] ; then
BACKUP_PART_NO=${BACKUP_PART_BIOS}
else
BACKUP_PART_NO=${BACKUP_PART_UEFI}
fi
BACKUP_PART_LABEL="platform_backup"
BACKUP_PART_GUID="BA5EBA11-0000-1111-2222-000000000002"
fi
TRAIT__CONTROLLER="controller"
TRAIT__STORAGE="storage"
TRAIT__WORKER="worker"
TRAIT__LOWLATENCY="lowlatency"
SYSTEM_TYPE__AIO="All-in-one"
SYSTEM_TYPE__STANDARD="Standard"
SUBFUNCTION__CONTROLLER="${TRAIT__CONTROLLER}"
SUBFUNCTION__STORAGE="${TRAIT__STORAGE}"
SUBFUNCTION__WORKER="${TRAIT__WORKER}"
SUBFUNCTION__AIO="${TRAIT__CONTROLLER},${TRAIT__WORKER}"
VOLUME_GROUPS="cgts-vg"
